@@ -1,19 +1,20 @@
 /*
- * Copyright (C) 1999-2009, 2011-2016  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ *
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
-/* $Id$ */
-
 #ifndef DNS_DB_H
 #define DNS_DB_H 1
 
 /*****
- ***** Module Info
- *****/
+***** Module Info
+*****/
 
 /*! \file dns/db.h
  * \brief
@@ -42,12 +43,15 @@
  */
 
 /*****
- ***** Imports
- *****/
+***** Imports
+*****/
 
+#include <inttypes.h>
+#include <stdbool.h>
+
+#include <isc/deprecated.h>
 #include <isc/lang.h>
 #include <isc/magic.h>
-#include <isc/ondestroy.h>
 #include <isc/stats.h>
 #include <isc/stdtime.h>
 
@@ -56,153 +60,138 @@
 #include <dns/name.h>
 #include <dns/rdata.h>
 #include <dns/rdataset.h>
-#include <dns/rpz.h>
 #include <dns/types.h>
 
 ISC_LANG_BEGINDECLS
 
 /*****
- ***** Types
- *****/
+***** Types
+*****/
 
 typedef struct dns_dbmethods {
-	void		(*attach)(dns_db_t *source, dns_db_t **targetp);
-	void		(*detach)(dns_db_t **dbp);
-	isc_result_t	(*beginload)(dns_db_t *db,
-				     dns_rdatacallbacks_t *callbacks);
-	isc_result_t	(*endload)(dns_db_t *db,
-				     dns_rdatacallbacks_t *callbacks);
-	isc_result_t	(*serialize)(dns_db_t *db,
-				     dns_dbversion_t *version, FILE *file);
-	isc_result_t	(*dump)(dns_db_t *db, dns_dbversion_t *version,
-				const char *filename,
-				dns_masterformat_t masterformat);
-	void		(*currentversion)(dns_db_t *db,
-					  dns_dbversion_t **versionp);
-	isc_result_t	(*newversion)(dns_db_t *db,
-				      dns_dbversion_t **versionp);
-	void		(*attachversion)(dns_db_t *db, dns_dbversion_t *source,
-					 dns_dbversion_t **targetp);
-	void		(*closeversion)(dns_db_t *db,
-					dns_dbversion_t **versionp,
-					isc_boolean_t commit);
-	isc_result_t	(*findnode)(dns_db_t *db, dns_name_t *name,
-				    isc_boolean_t create,
-				    dns_dbnode_t **nodep);
-	isc_result_t	(*find)(dns_db_t *db, dns_name_t *name,
-				dns_dbversion_t *version,
-				dns_rdatatype_t type, unsigned int options,
-				isc_stdtime_t now,
-				dns_dbnode_t **nodep, dns_name_t *foundname,
-				dns_rdataset_t *rdataset,
-				dns_rdataset_t *sigrdataset);
-	isc_result_t	(*findzonecut)(dns_db_t *db, dns_name_t *name,
-				       unsigned int options, isc_stdtime_t now,
-				       dns_dbnode_t **nodep,
-				       dns_name_t *foundname,
-				       dns_rdataset_t *rdataset,
-				       dns_rdataset_t *sigrdataset);
-	void		(*attachnode)(dns_db_t *db,
-				      dns_dbnode_t *source,
-				      dns_dbnode_t **targetp);
-	void		(*detachnode)(dns_db_t *db,
-				      dns_dbnode_t **targetp);
-	isc_result_t	(*expirenode)(dns_db_t *db, dns_dbnode_t *node,
-				      isc_stdtime_t now);
-	void		(*printnode)(dns_db_t *db, dns_dbnode_t *node,
-				     FILE *out);
-	isc_result_t 	(*createiterator)(dns_db_t *db, unsigned int options,
-					  dns_dbiterator_t **iteratorp);
-	isc_result_t	(*findrdataset)(dns_db_t *db, dns_dbnode_t *node,
-					dns_dbversion_t *version,
-					dns_rdatatype_t type,
-					dns_rdatatype_t covers,
-					isc_stdtime_t now,
-					dns_rdataset_t *rdataset,
-					dns_rdataset_t *sigrdataset);
-	isc_result_t	(*allrdatasets)(dns_db_t *db, dns_dbnode_t *node,
-					dns_dbversion_t *version,
-					isc_stdtime_t now,
-					dns_rdatasetiter_t **iteratorp);
-	isc_result_t	(*addrdataset)(dns_db_t *db, dns_dbnode_t *node,
+	void (*attach)(dns_db_t *source, dns_db_t **targetp);
+	void (*detach)(dns_db_t **dbp);
+	isc_result_t (*beginload)(dns_db_t *		db,
+				  dns_rdatacallbacks_t *callbacks);
+	isc_result_t (*endload)(dns_db_t *db, dns_rdatacallbacks_t *callbacks);
+	isc_result_t (*serialize)(dns_db_t *db, dns_dbversion_t *version,
+				  FILE *file);
+	isc_result_t (*dump)(dns_db_t *db, dns_dbversion_t *version,
+			     const char *	filename,
+			     dns_masterformat_t masterformat);
+	void (*currentversion)(dns_db_t *db, dns_dbversion_t **versionp);
+	isc_result_t (*newversion)(dns_db_t *db, dns_dbversion_t **versionp);
+	void (*attachversion)(dns_db_t *db, dns_dbversion_t *source,
+			      dns_dbversion_t **targetp);
+	void (*closeversion)(dns_db_t *db, dns_dbversion_t **versionp,
+			     bool commit);
+	isc_result_t (*findnode)(dns_db_t *db, const dns_name_t *name,
+				 bool create, dns_dbnode_t **nodep);
+	isc_result_t (*find)(dns_db_t *db, const dns_name_t *name,
+			     dns_dbversion_t *version, dns_rdatatype_t type,
+			     unsigned int options, isc_stdtime_t now,
+			     dns_dbnode_t **nodep, dns_name_t *foundname,
+			     dns_rdataset_t *rdataset,
+			     dns_rdataset_t *sigrdataset);
+	isc_result_t (*findzonecut)(dns_db_t *db, const dns_name_t *name,
+				    unsigned int options, isc_stdtime_t now,
+				    dns_dbnode_t **nodep, dns_name_t *foundname,
+				    dns_name_t *    dcname,
+				    dns_rdataset_t *rdataset,
+				    dns_rdataset_t *sigrdataset);
+	void (*attachnode)(dns_db_t *db, dns_dbnode_t *source,
+			   dns_dbnode_t **targetp);
+	void (*detachnode)(dns_db_t *db, dns_dbnode_t **targetp);
+	isc_result_t (*expirenode)(dns_db_t *db, dns_dbnode_t *node,
+				   isc_stdtime_t now);
+	void (*printnode)(dns_db_t *db, dns_dbnode_t *node, FILE *out);
+	isc_result_t (*createiterator)(dns_db_t *db, unsigned int options,
+				       dns_dbiterator_t **iteratorp);
+	isc_result_t (*findrdataset)(dns_db_t *db, dns_dbnode_t *node,
+				     dns_dbversion_t *version,
+				     dns_rdatatype_t  type,
+				     dns_rdatatype_t covers, isc_stdtime_t now,
+				     dns_rdataset_t *rdataset,
+				     dns_rdataset_t *sigrdataset);
+	isc_result_t (*allrdatasets)(dns_db_t *db, dns_dbnode_t *node,
+				     dns_dbversion_t *	  version,
+				     isc_stdtime_t	  now,
+				     dns_rdatasetiter_t **iteratorp);
+	isc_result_t (*addrdataset)(dns_db_t *db, dns_dbnode_t *node,
+				    dns_dbversion_t *version, isc_stdtime_t now,
+				    dns_rdataset_t *rdataset,
+				    unsigned int    options,
+				    dns_rdataset_t *addedrdataset);
+	isc_result_t (*subtractrdataset)(dns_db_t *db, dns_dbnode_t *node,
+					 dns_dbversion_t *version,
+					 dns_rdataset_t * rdataset,
+					 unsigned int	  options,
+					 dns_rdataset_t * newrdataset);
+	isc_result_t (*deleterdataset)(dns_db_t *db, dns_dbnode_t *node,
 				       dns_dbversion_t *version,
-				       isc_stdtime_t now,
-				       dns_rdataset_t *rdataset,
-				       unsigned int options,
-				       dns_rdataset_t *addedrdataset);
-	isc_result_t	(*subtractrdataset)(dns_db_t *db, dns_dbnode_t *node,
-					    dns_dbversion_t *version,
-					    dns_rdataset_t *rdataset,
-					    unsigned int options,
-					    dns_rdataset_t *newrdataset);
-	isc_result_t	(*deleterdataset)(dns_db_t *db, dns_dbnode_t *node,
-					  dns_dbversion_t *version,
-					  dns_rdatatype_t type,
-					  dns_rdatatype_t covers);
-	isc_boolean_t	(*issecure)(dns_db_t *db);
-	unsigned int	(*nodecount)(dns_db_t *db);
-	isc_boolean_t	(*ispersistent)(dns_db_t *db);
-	void		(*overmem)(dns_db_t *db, isc_boolean_t overmem);
-	void		(*settask)(dns_db_t *db, isc_task_t *);
-	isc_result_t	(*getoriginnode)(dns_db_t *db, dns_dbnode_t **nodep);
-	void		(*transfernode)(dns_db_t *db, dns_dbnode_t **sourcep,
-					dns_dbnode_t **targetp);
-	isc_result_t    (*getnsec3parameters)(dns_db_t *db,
-					      dns_dbversion_t *version,
-					      dns_hash_t *hash,
-					      isc_uint8_t *flags,
-					      isc_uint16_t *iterations,
-					      unsigned char *salt,
-					      size_t *salt_len);
-	isc_result_t    (*findnsec3node)(dns_db_t *db, dns_name_t *name,
-					 isc_boolean_t create,
-					 dns_dbnode_t **nodep);
-	isc_result_t	(*setsigningtime)(dns_db_t *db,
-					  dns_rdataset_t *rdataset,
-					  isc_stdtime_t resign);
-	isc_result_t	(*getsigningtime)(dns_db_t *db,
-					  dns_rdataset_t *rdataset,
-					  dns_name_t *name);
-	void		(*resigned)(dns_db_t *db, dns_rdataset_t *rdataset,
-					   dns_dbversion_t *version);
-	isc_boolean_t	(*isdnssec)(dns_db_t *db);
-	dns_stats_t	*(*getrrsetstats)(dns_db_t *db);
-	void		(*rpz_attach)(dns_db_t *db, dns_rpz_zones_t *rpzs,
-				      dns_rpz_num_t rpz_num);
-	isc_result_t	(*rpz_ready)(dns_db_t *db);
-	isc_result_t	(*findnodeext)(dns_db_t *db, dns_name_t *name,
-				     isc_boolean_t create,
-				     dns_clientinfomethods_t *methods,
-				     dns_clientinfo_t *clientinfo,
-				     dns_dbnode_t **nodep);
-	isc_result_t	(*findext)(dns_db_t *db, dns_name_t *name,
-				   dns_dbversion_t *version,
-				   dns_rdatatype_t type, unsigned int options,
-				   isc_stdtime_t now,
-				   dns_dbnode_t **nodep, dns_name_t *foundname,
-				   dns_clientinfomethods_t *methods,
-				   dns_clientinfo_t *clientinfo,
-				   dns_rdataset_t *rdataset,
-				   dns_rdataset_t *sigrdataset);
-	isc_result_t	(*setcachestats)(dns_db_t *db, isc_stats_t *stats);
-	size_t		(*hashsize)(dns_db_t *db);
-	isc_result_t	(*nodefullname)(dns_db_t *db, dns_dbnode_t *node,
-					dns_name_t *name);
-	isc_result_t	(*getsize)(dns_db_t *db, dns_dbversion_t *version,
-				   isc_uint64_t *records, isc_uint64_t *bytes);
+				       dns_rdatatype_t	type,
+				       dns_rdatatype_t	covers);
+	bool (*issecure)(dns_db_t *db);
+	unsigned int (*nodecount)(dns_db_t *db);
+	bool (*ispersistent)(dns_db_t *db);
+	void (*overmem)(dns_db_t *db, bool overmem);
+	void (*settask)(dns_db_t *db, isc_task_t *);
+	isc_result_t (*getoriginnode)(dns_db_t *db, dns_dbnode_t **nodep);
+	void (*transfernode)(dns_db_t *db, dns_dbnode_t **sourcep,
+			     dns_dbnode_t **targetp);
+	isc_result_t (*getnsec3parameters)(dns_db_t *	    db,
+					   dns_dbversion_t *version,
+					   dns_hash_t *hash, uint8_t *flags,
+					   uint16_t *	  iterations,
+					   unsigned char *salt,
+					   size_t *	  salt_len);
+	isc_result_t (*findnsec3node)(dns_db_t *db, const dns_name_t *name,
+				      bool create, dns_dbnode_t **nodep);
+	isc_result_t (*setsigningtime)(dns_db_t *db, dns_rdataset_t *rdataset,
+				       isc_stdtime_t resign);
+	isc_result_t (*getsigningtime)(dns_db_t *db, dns_rdataset_t *rdataset,
+				       dns_name_t *name);
+	void (*resigned)(dns_db_t *db, dns_rdataset_t *rdataset,
+			 dns_dbversion_t *version);
+	bool (*isdnssec)(dns_db_t *db);
+	dns_stats_t *(*getrrsetstats)(dns_db_t *db);
+	void (*rpz_attach)(dns_db_t *db, void *rpzs, uint8_t rpz_num);
+	isc_result_t (*rpz_ready)(dns_db_t *db);
+	isc_result_t (*findnodeext)(dns_db_t *db, const dns_name_t *name,
+				    bool		     create,
+				    dns_clientinfomethods_t *methods,
+				    dns_clientinfo_t *	     clientinfo,
+				    dns_dbnode_t **	     nodep);
+	isc_result_t (*findext)(dns_db_t *db, const dns_name_t *name,
+				dns_dbversion_t *version, dns_rdatatype_t type,
+				unsigned int options, isc_stdtime_t now,
+				dns_dbnode_t **nodep, dns_name_t *foundname,
+				dns_clientinfomethods_t *methods,
+				dns_clientinfo_t *	 clientinfo,
+				dns_rdataset_t *	 rdataset,
+				dns_rdataset_t *	 sigrdataset);
+	isc_result_t (*setcachestats)(dns_db_t *db, isc_stats_t *stats);
+	size_t (*hashsize)(dns_db_t *db);
+	isc_result_t (*nodefullname)(dns_db_t *db, dns_dbnode_t *node,
+				     dns_name_t *name);
+	isc_result_t (*getsize)(dns_db_t *db, dns_dbversion_t *version,
+				uint64_t *records, uint64_t *bytes);
+	isc_result_t (*setservestalettl)(dns_db_t *db, dns_ttl_t ttl);
+	isc_result_t (*getservestalettl)(dns_db_t *db, dns_ttl_t *ttl);
+	isc_result_t (*setgluecachestats)(dns_db_t *db, isc_stats_t *stats);
 } dns_dbmethods_t;
 
-typedef isc_result_t
-(*dns_dbcreatefunc_t)(isc_mem_t *mctx, dns_name_t *name,
-		      dns_dbtype_t type, dns_rdataclass_t rdclass,
-		      unsigned int argc, char *argv[], void *driverarg,
-		      dns_db_t **dbp);
+typedef isc_result_t (*dns_dbcreatefunc_t)(isc_mem_t *	     mctx,
+					   const dns_name_t *name,
+					   dns_dbtype_t	     type,
+					   dns_rdataclass_t  rdclass,
+					   unsigned int argc, char *argv[],
+					   void *driverarg, dns_db_t **dbp);
 
-typedef isc_result_t
-(*dns_dbupdate_callback_t)(dns_db_t *db, void *fn_arg);
+typedef isc_result_t (*dns_dbupdate_callback_t)(dns_db_t *db, void *fn_arg);
 
-#define DNS_DB_MAGIC		ISC_MAGIC('D','N','S','D')
-#define DNS_DB_VALID(db)	ISC_MAGIC_VALID(db, DNS_DB_MAGIC)
+#define DNS_DB_MAGIC	 ISC_MAGIC('D', 'N', 'S', 'D')
+#define DNS_DB_VALID(db) ISC_MAGIC_VALID(db, DNS_DB_MAGIC)
 
 /*%
  * This structure is actually just the common prefix of a DNS db
@@ -214,80 +203,80 @@ typedef isc_result_t
  * invariants.
  */
 struct dns_db {
-	unsigned int				magic;
-	unsigned int				impmagic;
-	dns_dbmethods_t *			methods;
-	isc_uint16_t				attributes;
-	dns_rdataclass_t			rdclass;
-	dns_name_t				origin;
-	isc_ondestroy_t				ondest;
-	isc_mem_t *				mctx;
-	ISC_LIST(dns_dbonupdatelistener_t)	update_listeners;
+	unsigned int	 magic;
+	unsigned int	 impmagic;
+	dns_dbmethods_t *methods;
+	uint16_t	 attributes;
+	dns_rdataclass_t rdclass;
+	dns_name_t	 origin;
+	isc_mem_t *	 mctx;
+	ISC_LIST(dns_dbonupdatelistener_t) update_listeners;
 };
 
-#define DNS_DBATTR_CACHE		0x01
-#define DNS_DBATTR_STUB			0x02
+#define DNS_DBATTR_CACHE 0x01
+#define DNS_DBATTR_STUB	 0x02
 
 struct dns_dbonupdatelistener {
-	dns_dbupdate_callback_t			onupdate;
-	void *					onupdate_arg;
-	ISC_LINK(dns_dbonupdatelistener_t)	link;
+	dns_dbupdate_callback_t onupdate;
+	void *			onupdate_arg;
+	ISC_LINK(dns_dbonupdatelistener_t) link;
 };
 
 /*@{*/
 /*%
  * Options that can be specified for dns_db_find().
  */
-#define DNS_DBFIND_GLUEOK		0x0001
-#define DNS_DBFIND_VALIDATEGLUE		0x0002
-#define DNS_DBFIND_NOWILD		0x0004
-#define DNS_DBFIND_PENDINGOK		0x0008
-#define DNS_DBFIND_NOEXACT		0x0010
-#define DNS_DBFIND_FORCENSEC		0x0020
-#define DNS_DBFIND_COVERINGNSEC		0x0040
-#define DNS_DBFIND_FORCENSEC3		0x0080
-#define DNS_DBFIND_ADDITIONALOK		0x0100
-#define DNS_DBFIND_NOZONECUT		0x0200
+#define DNS_DBFIND_GLUEOK	0x0001
+#define DNS_DBFIND_VALIDATEGLUE 0x0002
+#define DNS_DBFIND_NOWILD	0x0004
+#define DNS_DBFIND_PENDINGOK	0x0008
+#define DNS_DBFIND_NOEXACT	0x0010
+#define DNS_DBFIND_FORCENSEC	0x0020
+#define DNS_DBFIND_COVERINGNSEC 0x0040
+#define DNS_DBFIND_FORCENSEC3	0x0080
+#define DNS_DBFIND_ADDITIONALOK 0x0100
+#define DNS_DBFIND_NOZONECUT	0x0200
+#define DNS_DBFIND_STALEOK	0x0400
 /*@}*/
 
 /*@{*/
 /*%
  * Options that can be specified for dns_db_addrdataset().
  */
-#define DNS_DBADD_MERGE			0x01
-#define DNS_DBADD_FORCE			0x02
-#define DNS_DBADD_EXACT			0x04
-#define DNS_DBADD_EXACTTTL		0x08
-#define DNS_DBADD_PREFETCH		0x10
+#define DNS_DBADD_MERGE	   0x01
+#define DNS_DBADD_FORCE	   0x02
+#define DNS_DBADD_EXACT	   0x04
+#define DNS_DBADD_EXACTTTL 0x08
+#define DNS_DBADD_PREFETCH 0x10
 /*@}*/
 
 /*%
  * Options that can be specified for dns_db_subtractrdataset().
  */
-#define DNS_DBSUB_EXACT			0x01
-#define DNS_DBSUB_WANTOLD		0x02
+#define DNS_DBSUB_EXACT	  0x01
+#define DNS_DBSUB_WANTOLD 0x02
 
 /*@{*/
 /*%
  * Iterator options
  */
-#define DNS_DB_RELATIVENAMES	0x1
-#define DNS_DB_NSEC3ONLY	0x2
-#define DNS_DB_NONSEC3		0x4
+#define DNS_DB_RELATIVENAMES 0x1
+#define DNS_DB_NSEC3ONLY     0x2
+#define DNS_DB_NONSEC3	     0x4
 /*@}*/
 
 /*****
- ***** Methods
- *****/
+***** Methods
+*****/
 
 /***
  *** Basic DB Methods
  ***/
 
 isc_result_t
-dns_db_create(isc_mem_t *mctx, const char *db_type, dns_name_t *origin,
-	      dns_dbtype_t type, dns_rdataclass_t rdclass,
-	      unsigned int argc, char *argv[], dns_db_t **dbp);
+dns_db_create(isc_mem_t *mctx, const char *db_type, const dns_name_t *origin,
+	      dns_dbtype_t type, dns_rdataclass_t rdclass, unsigned int argc,
+	      char *argv[], dns_db_t **dbp);
 /*%<
  * Create a new database using implementation 'db_type'.
  *
@@ -356,18 +345,7 @@ dns_db_detach(dns_db_t **dbp);
  *		all resources used by the database will be freed
  */
 
-isc_result_t
-dns_db_ondestroy(dns_db_t *db, isc_task_t *task, isc_event_t **eventp);
-/*%<
- * Causes 'eventp' to be sent to be sent to 'task' when the database is
- * destroyed.
- *
- * Note; ownership of the eventp is taken from the caller (and *eventp is
- * set to NULL). The sender field of the event is set to 'db' before it is
- * sent to the task.
- */
-
-isc_boolean_t
+bool
 dns_db_iscache(dns_db_t *db);
 /*%<
  * Does 'db' have cache semantics?
@@ -377,11 +355,11 @@ dns_db_iscache(dns_db_t *db);
  * \li	'db' is a valid database.
  *
  * Returns:
- * \li	#ISC_TRUE	'db' has cache semantics
- * \li	#ISC_FALSE	otherwise
+ * \li	#true	'db' has cache semantics
+ * \li	#false	otherwise
  */
 
-isc_boolean_t
+bool
 dns_db_iszone(dns_db_t *db);
 /*%<
  * Does 'db' have zone semantics?
@@ -391,11 +369,11 @@ dns_db_iszone(dns_db_t *db);
  * \li	'db' is a valid database.
  *
  * Returns:
- * \li	#ISC_TRUE	'db' has zone semantics
- * \li	#ISC_FALSE	otherwise
+ * \li	#true	'db' has zone semantics
+ * \li	#false	otherwise
  */
 
-isc_boolean_t
+bool
 dns_db_isstub(dns_db_t *db);
 /*%<
  * Does 'db' have stub semantics?
@@ -405,11 +383,11 @@ dns_db_isstub(dns_db_t *db);
  * \li	'db' is a valid database.
  *
  * Returns:
- * \li	#ISC_TRUE	'db' has zone semantics
- * \li	#ISC_FALSE	otherwise
+ * \li	#true	'db' has zone semantics
+ * \li	#false	otherwise
  */
 
-isc_boolean_t
+bool
 dns_db_issecure(dns_db_t *db);
 /*%<
  * Is 'db' secure?
@@ -419,11 +397,11 @@ dns_db_issecure(dns_db_t *db);
  * \li	'db' is a valid database with zone semantics.
  *
  * Returns:
- * \li	#ISC_TRUE	'db' is secure.
- * \li	#ISC_FALSE	'db' is not secure.
+ * \li	#true	'db' is secure.
+ * \li	#false	'db' is not secure.
  */
 
-isc_boolean_t
+bool
 dns_db_isdnssec(dns_db_t *db);
 /*%<
  * Is 'db' secure or partially secure?
@@ -433,8 +411,8 @@ dns_db_isdnssec(dns_db_t *db);
  * \li	'db' is a valid database with zone semantics.
  *
  * Returns:
- * \li	#ISC_TRUE	'db' is secure or is partially.
- * \li	#ISC_FALSE	'db' is not secure.
+ * \li	#true	'db' is secure or is partially.
+ * \li	#false	'db' is not secure.
  */
 
 dns_name_t *
@@ -526,14 +504,8 @@ dns_db_endload(dns_db_t *db, dns_rdatacallbacks_t *callbacks);
  */
 
 isc_result_t
-dns_db_load(dns_db_t *db, const char *filename);
-
-isc_result_t
-dns_db_load2(dns_db_t *db, const char *filename, dns_masterformat_t format);
-
-isc_result_t
-dns_db_load3(dns_db_t *db, const char *filename, dns_masterformat_t format,
-	     unsigned int options);
+dns_db_load(dns_db_t *db, const char *filename, dns_masterformat_t format,
+	    unsigned int options);
 /*%<
  * Load master file 'filename' into 'db'.
  *
@@ -583,10 +555,6 @@ dns_db_serialize(dns_db_t *db, dns_dbversion_t *version, FILE *rbtfile);
 
 isc_result_t
 dns_db_dump(dns_db_t *db, dns_dbversion_t *version, const char *filename);
-
-isc_result_t
-dns_db_dump2(dns_db_t *db, dns_dbversion_t *version, const char *filename,
-	     dns_masterformat_t masterformat);
 /*%<
  * Dump version 'version' of 'db' to master file 'filename'.
  *
@@ -670,12 +638,11 @@ dns_db_attachversion(dns_db_t *db, dns_dbversion_t *source,
  */
 
 void
-dns_db_closeversion(dns_db_t *db, dns_dbversion_t **versionp,
-		    isc_boolean_t commit);
+dns_db_closeversion(dns_db_t *db, dns_dbversion_t **versionp, bool commit);
 /*%<
  * Close version '*versionp'.
  *
- * Note: if '*versionp' is a read-write version and 'commit' is ISC_TRUE,
+ * Note: if '*versionp' is a read-write version and 'commit' is true,
  * then all changes made in the version will take effect, otherwise they
  * will be rolled back.  The value of 'commit' is ignored for read-only
  * versions.
@@ -694,7 +661,7 @@ dns_db_closeversion(dns_db_t *db, dns_dbversion_t **versionp,
  *
  * \li	*versionp == NULL
  *
- * \li	If *versionp is a read-write version, and commit is ISC_TRUE, then
+ * \li	If *versionp is a read-write version, and commit is true, then
  *	the version will become the current version.  If !commit, then all
  *	changes made in the version will be undone, and the version will
  *	not become the current version.
@@ -705,11 +672,11 @@ dns_db_closeversion(dns_db_t *db, dns_dbversion_t **versionp,
  ***/
 
 isc_result_t
-dns_db_findnode(dns_db_t *db, dns_name_t *name, isc_boolean_t create,
+dns_db_findnode(dns_db_t *db, const dns_name_t *name, bool create,
 		dns_dbnode_t **nodep);
 
 isc_result_t
-dns_db_findnodeext(dns_db_t *db, dns_name_t *name, isc_boolean_t create,
+dns_db_findnodeext(dns_db_t *db, const dns_name_t *name, bool create,
 		   dns_clientinfomethods_t *methods,
 		   dns_clientinfo_t *clientinfo, dns_dbnode_t **nodep);
 /*%<
@@ -717,11 +684,11 @@ dns_db_findnodeext(dns_db_t *db, dns_name_t *name, isc_boolean_t create,
  *
  * dns_db_findnodeext() (findnode extended) also accepts parameters
  * 'methods' and 'clientinfo', which, when provided, enable the database to
- * retreive information about the client from the caller, and modify its
+ * retrieve information about the client from the caller, and modify its
  * response on the basis of that information.
  *
  * Notes:
- * \li	If 'create' is ISC_TRUE and no node with name 'name' exists, then
+ * \li	If 'create' is true and no node with name 'name' exists, then
  *	such a node will be created.
  *
  * \li	This routine is for finding or creating a node with the specified
@@ -745,20 +712,20 @@ dns_db_findnodeext(dns_db_t *db, dns_name_t *name, isc_boolean_t create,
  *
  * \li	#ISC_R_SUCCESS
  * \li	#ISC_R_NOTFOUND			If !create and name not found.
- * \li	#ISC_R_NOMEMORY			Can only happen if create is ISC_TRUE.
+ * \li	#ISC_R_NOMEMORY			Can only happen if create is true.
  *
  * \li	Other results are possible, depending upon the database
  *	implementation used.
  */
 
 isc_result_t
-dns_db_find(dns_db_t *db, dns_name_t *name, dns_dbversion_t *version,
+dns_db_find(dns_db_t *db, const dns_name_t *name, dns_dbversion_t *version,
 	    dns_rdatatype_t type, unsigned int options, isc_stdtime_t now,
 	    dns_dbnode_t **nodep, dns_name_t *foundname,
 	    dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset);
 
 isc_result_t
-dns_db_findext(dns_db_t *db, dns_name_t *name, dns_dbversion_t *version,
+dns_db_findext(dns_db_t *db, const dns_name_t *name, dns_dbversion_t *version,
 	       dns_rdatatype_t type, unsigned int options, isc_stdtime_t now,
 	       dns_dbnode_t **nodep, dns_name_t *foundname,
 	       dns_clientinfomethods_t *methods, dns_clientinfo_t *clientinfo,
@@ -767,7 +734,7 @@ dns_db_findext(dns_db_t *db, dns_name_t *name, dns_dbversion_t *version,
  * Find the best match for 'name' and 'type' in version 'version' of 'db'.
  *
  * dns_db_findext() (find extended) also accepts parameters 'methods'
- * and 'clientinfo', which when provided enable the database to retreive
+ * and 'clientinfo', which when provided enable the database to retrieve
  * information about the client from the caller, and modify its response
  * on the basis of this information.
  *
@@ -856,14 +823,6 @@ dns_db_findext(dns_db_t *db, dns_name_t *name, dns_dbversion_t *version,
  *	\li	#ISC_R_SUCCESS			The desired node and type were
  *						found.
  *
- *	\li	#DNS_R_WILDCARD			The desired node and type were
- *						found after performing
- *						wildcard matching.  This is
- *						only returned if the
- *						#DNS_DBFIND_INDICATEWILD
- *						option is set; otherwise
- *						#ISC_R_SUCCESS is returned.
- *
  *	\li	#DNS_R_GLUE			The desired node and type were
  *						found, but are glue.  This
  *						result can only occur if
@@ -956,9 +915,9 @@ dns_db_findext(dns_db_t *db, dns_name_t *name, dns_dbversion_t *version,
  */
 
 isc_result_t
-dns_db_findzonecut(dns_db_t *db, dns_name_t *name,
-		   unsigned int options, isc_stdtime_t now,
-		   dns_dbnode_t **nodep, dns_name_t *foundname,
+dns_db_findzonecut(dns_db_t *db, const dns_name_t *name, unsigned int options,
+		   isc_stdtime_t now, dns_dbnode_t **nodep,
+		   dns_name_t *foundname, dns_name_t *dcname,
 		   dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset);
 /*%<
  * Find the deepest known zonecut which encloses 'name' in 'db'.
@@ -978,6 +937,8 @@ dns_db_findzonecut(dns_db_t *db, dns_name_t *name,
  *
  * \li	'foundname' is a valid name with a dedicated buffer.
  *
+ * \li	'dcname' is a valid name with a dedicated buffer.
+ *
  * \li	'rdataset' is NULL, or is a valid unassociated rdataset.
  *
  * Ensures, on a non-error completion:
@@ -987,6 +948,9 @@ dns_db_findzonecut(dns_db_t *db, dns_name_t *name,
  * \li	If foundname != NULL, then it contains the full name of the
  *	found node.
  *
+ * \li	If dcname != NULL, then it contains the deepest cached name
+ *      that exists in the database.
+ *
  * \li	If rdataset != NULL and type != dns_rdatatype_any, then
  *	rdataset is bound to the found rdataset.
  *
@@ -1158,7 +1122,7 @@ dns_db_findrdataset(dns_db_t *db, dns_dbnode_t *node, dns_dbversion_t *version,
  *
  * \li	'sigrdataset' is a valid, disassociated rdataset, or it is NULL.
  *
- * \li	If 'covers' != 0, 'type' must be SIG.
+ * \li	If 'covers' != 0, 'type' must be RRSIG.
  *
  * \li	'type' is not a meta-RR type such as 'ANY' or 'OPT'.
  *
@@ -1264,9 +1228,8 @@ dns_db_addrdataset(dns_db_t *db, dns_dbnode_t *node, dns_dbversion_t *version,
  * Returns:
  *
  * \li	#ISC_R_SUCCESS
- * \li	#DNS_R_UNCHANGED			The operation did not change anything.
- * \li	#ISC_R_NOMEMORY
- * \li	#DNS_R_NOTEXACT
+ * \li	#DNS_R_UNCHANGED			The operation did not change
+ * anything. \li	#ISC_R_NOMEMORY \li	#DNS_R_NOTEXACT
  *
  * \li	Other results are possible, depending upon the database
  *	implementation used.
@@ -1307,11 +1270,11 @@ dns_db_subtractrdataset(dns_db_t *db, dns_dbnode_t *node,
  * Returns:
  *
  * \li	#ISC_R_SUCCESS
- * \li	#DNS_R_UNCHANGED			The operation did not change anything.
- * \li	#DNS_R_NXRRSET			All rdata of the same type as those
- *					in 'rdataset' have been deleted.
- * \li	#DNS_R_NOTEXACT			Some part of 'rdataset' did not
- *					exist and DNS_DBSUB_EXACT was set.
+ * \li	#DNS_R_UNCHANGED			The operation did not change
+ * anything. \li	#DNS_R_NXRRSET			All rdata of the same
+ *type as
+ * those in 'rdataset' have been deleted. \li	#DNS_R_NOTEXACT
+ * Some part of 'rdataset' did not exist and DNS_DBSUB_EXACT was set.
  *
  * \li	Other results are possible, depending upon the database
  *	implementation used.
@@ -1349,15 +1312,15 @@ dns_db_deleterdataset(dns_db_t *db, dns_dbnode_t *node,
  * Returns:
  *
  * \li	#ISC_R_SUCCESS
- * \li	#DNS_R_UNCHANGED			No rdatasets of 'type' existed before
- *					the operation was attempted.
+ * \li	#DNS_R_UNCHANGED			No rdatasets of 'type' existed
+ * before the operation was attempted.
  *
  * \li	Other results are possible, depending upon the database
  *	implementation used.
  */
 
 isc_result_t
-dns_db_getsoaserial(dns_db_t *db, dns_dbversion_t *ver, isc_uint32_t *serialp);
+dns_db_getsoaserial(dns_db_t *db, dns_dbversion_t *ver, uint32_t *serialp);
 /*%<
  * Get the current SOA serial number from a zone database.
  *
@@ -1367,7 +1330,7 @@ dns_db_getsoaserial(dns_db_t *db, dns_dbversion_t *ver, isc_uint32_t *serialp);
  */
 
 void
-dns_db_overmem(dns_db_t *db, isc_boolean_t overmem);
+dns_db_overmem(dns_db_t *db, bool overmem);
 /*%<
  * Enable / disable aggressive cache cleaning.
  */
@@ -1410,7 +1373,7 @@ dns_db_settask(dns_db_t *db, isc_task_t *task);
  * \li	'task' to be valid or NULL.
  */
 
-isc_boolean_t
+bool
 dns_db_ispersistent(dns_db_t *db);
 /*%<
  * Is 'db' persistent?  A persistent database does not need to be loaded
@@ -1421,8 +1384,8 @@ dns_db_ispersistent(dns_db_t *db);
  * \li	'db' is a valid database.
  *
  * Returns:
- * \li	#ISC_TRUE	'db' is persistent.
- * \li	#ISC_FALSE	'db' is not persistent.
+ * \li	#true	'db' is persistent.
+ * \li	#false	'db' is not persistent.
  */
 
 isc_result_t
@@ -1488,9 +1451,9 @@ dns_db_getoriginnode(dns_db_t *db, dns_dbnode_t **nodep);
 
 isc_result_t
 dns_db_getnsec3parameters(dns_db_t *db, dns_dbversion_t *version,
-			  dns_hash_t *hash, isc_uint8_t *flags,
-			  isc_uint16_t *interations,
-			  unsigned char *salt, size_t *salt_length);
+			  dns_hash_t *hash, uint8_t *flags,
+			  uint16_t *iterations, unsigned char *salt,
+			  size_t *salt_length);
 /*%<
  * Get the NSEC3 parameters that are associated with this zone.
  *
@@ -1504,11 +1467,15 @@ dns_db_getnsec3parameters(dns_db_t *db, dns_dbversion_t *version,
  */
 
 isc_result_t
-dns_db_getsize(dns_db_t *db, dns_dbversion_t *version, isc_uint64_t *records,
-	       isc_uint64_t *bytes);
+dns_db_getsize(dns_db_t *db, dns_dbversion_t *version, uint64_t *records,
+	       uint64_t *bytes);
 /*%<
- * Get the number of records in the given version of the database as well
- * as the number bytes used to store those records.
+ * On success if 'records' is not NULL, it is set to the number of records
+ * in the given version of the database. If 'bytes' is not NULL, it is
+ * set to the approximate number of bytes needed to transfer the records,
+ * counting name, TTL, type, class, and rdata for each RR.  (This is meant
+ * to be a rough approximation of the size of a full zone transfer, though
+ * it does not take into account DNS message overhead or name compression.)
  *
  * Requires:
  * \li	'db' is a valid zone database.
@@ -1522,13 +1489,13 @@ dns_db_getsize(dns_db_t *db, dns_dbversion_t *version, isc_uint64_t *records,
  */
 
 isc_result_t
-dns_db_findnsec3node(dns_db_t *db, dns_name_t *name,
-		     isc_boolean_t create, dns_dbnode_t **nodep);
+dns_db_findnsec3node(dns_db_t *db, const dns_name_t *name, bool create,
+		     dns_dbnode_t **nodep);
 /*%<
  * Find the NSEC3 node with name 'name'.
  *
  * Notes:
- * \li	If 'create' is ISC_TRUE and no node with name 'name' exists, then
+ * \li	If 'create' is true and no node with name 'name' exists, then
  *	such a node will be created.
  *
  * Requires:
@@ -1547,7 +1514,7 @@ dns_db_findnsec3node(dns_db_t *db, dns_name_t *name,
  *
  * \li	#ISC_R_SUCCESS
  * \li	#ISC_R_NOTFOUND			If !create and name not found.
- * \li	#ISC_R_NOMEMORY			Can only happen if create is ISC_TRUE.
+ * \li	#ISC_R_NOMEMORY			Can only happen if create is true.
  *
  * \li	Other results are possible, depending upon the database
  *	implementation used.
@@ -1635,21 +1602,20 @@ dns_db_setcachestats(dns_db_t *db, isc_stats_t *stats);
  */
 
 void
-dns_db_rpz_attach(dns_db_t *db, dns_rpz_zones_t *rpzs, dns_rpz_num_t rpz_num);
+dns_db_rpz_attach(dns_db_t *db, void *rpzs, uint8_t rpz_num) ISC_DEPRECATED;
 /*%<
  * Attach the response policy information for a view to a database for a
  * zone for the view.
  */
 
 isc_result_t
-dns_db_rpz_ready(dns_db_t *db);
+dns_db_rpz_ready(dns_db_t *db) ISC_DEPRECATED;
 /*%<
  * Finish loading a response policy zone.
  */
 
 isc_result_t
-dns_db_updatenotify_register(dns_db_t *db,
-			     dns_dbupdate_callback_t fn,
+dns_db_updatenotify_register(dns_db_t *db, dns_dbupdate_callback_t fn,
 			     void *fn_arg);
 /*%<
  * Register a notify-on-update callback function to a database.
@@ -1663,8 +1629,7 @@ dns_db_updatenotify_register(dns_db_t *db,
  */
 
 isc_result_t
-dns_db_updatenotify_unregister(dns_db_t *db,
-			       dns_dbupdate_callback_t fn,
+dns_db_updatenotify_unregister(dns_db_t *db, dns_dbupdate_callback_t fn,
 			       void *fn_arg);
 /*%<
  * Unregister a notify-on-update callback.
@@ -1686,6 +1651,53 @@ dns_db_nodefullname(dns_db_t *db, dns_dbnode_t *node, dns_name_t *name);
  * \li	'db' is a valid database
  * \li	'node' and 'name' are not NULL
  */
+
+isc_result_t
+dns_db_setservestalettl(dns_db_t *db, dns_ttl_t ttl);
+/*%<
+ * Sets the maximum length of time that cached answers may be retained
+ * past their normal TTL. Default value for the library is 0, disabling
+ * the use of stale data.
+ *
+ * Requires:
+ * \li	'db' is a valid cache database.
+ * \li	'ttl' is the number of seconds to retain data past its normal expiry.
+ *
+ * Returns:
+ * \li	#ISC_R_SUCCESS
+ * \li	#ISC_R_NOTIMPLEMENTED - Not supported by this DB implementation.
+ */
+
+isc_result_t
+dns_db_getservestalettl(dns_db_t *db, dns_ttl_t *ttl);
+/*%<
+ * Gets maximum length of time that cached answers may be kept past
+ * normal TTL expiration.
+ *
+ * Requires:
+ * \li	'db' is a valid cache database.
+ * \li	'ttl' is the number of seconds to retain data past its normal expiry.
+ *
+ * Returns:
+ * \li	#ISC_R_SUCCESS
+ * \li	#ISC_R_NOTIMPLEMENTED - Not supported by this DB implementation.
+ */
+
+isc_result_t
+dns_db_setgluecachestats(dns_db_t *db, isc_stats_t *stats);
+/*%<
+ * Set the location in which to collect glue cache statistics.
+ * This option may not exist depending on the DB implementation.
+ *
+ * Requires:
+ *
+ * \li	'db' is a valid database (cache only).
+ *
+ * Returns:
+ * \li	when available, a pointer to a statistics object created by
+ *	dns_rdatasetstats_create(); otherwise NULL.
+ */
+
 ISC_LANG_ENDDECLS
 
 #endif /* DNS_DB_H */
